import { __decorate } from "tslib";
import { css, customElement, FASTElement, observable } from '@microsoft/fast-element';
import { CriteriaBuilder } from '../criteria-builder';
import { ExpressionBuilder } from '../expression-builder';
import { Join } from '../types';
import { ensureArray } from '../utils';
import { template } from './criteria-segmented-control.template';
/**
 * Main class which defines the criteria segmented control component.
 *
 * @remarks
 *
 * Displays segmented criteria and dispatches events with the selected criteria.
 *
 * @public
 *
 * @example
 * Example of using the component:
 * ```
 *
 * const toolbarOptions: CriteriaSegmentedControlOption[] = [
 *   { label: 'A', field: 'INSTRUMENT_ID', value: 'id1', serialiser: Serialisers.EQ },
 *   { label: 'B', field: 'INSTRUMENT_ID', value: 'id2', serialiser: Serialisers.EQ },
 *   { label: 'C', field: 'INSTRUMENT_ID', value: 'id3', serialiser: Serialisers.EQ },
 * ];
 *
 *  <criteria-segmented-control
 *    :criteriaOptions=${() => toolbarOptions}
 *    :value=${sync((x) => x.criteriaFilter)}
 *  >
 *    <label slot="label">Filter by instrument name</label>
 *  </criteria-segmented-control>
 * ```
 *
 */
let CriteriaSegmentedControl = class CriteriaSegmentedControl extends FASTElement {
    constructor() {
        super(...arguments);
        this.criteriaOptions = [];
        this.createCriteria = (selectedOption) => {
            const criteriaBuilder = new CriteriaBuilder();
            const { field, serialiser, value } = selectedOption;
            if (Array.isArray(field) || Array.isArray(value)) {
                criteriaBuilder.And(ensureArray(value).map((option) => ensureArray(field).map((fname) => new ExpressionBuilder()
                    .withField(fname)
                    .withValue(option)
                    .withSerialiser(serialiser)
                    .build())), { join: Join.Or() });
            }
            else {
                criteriaBuilder.And(new ExpressionBuilder().withField(field).withValue(value).withSerialiser(serialiser).build());
            }
            return criteriaBuilder;
        };
    }
    valueChanged() {
        this.dispatchEvent(new Event('change', { bubbles: true, composed: true }));
    }
    selectedLabelChanged() {
        const selectedOption = this.criteriaOptions.find((option) => option.label === this.selectedLabel);
        if (selectedOption) {
            this.value = this.createCriteria(selectedOption).build();
        }
        else {
            this.value = '';
        }
    }
    itemClickHandler(label) {
        if (this.selectedLabel === label) {
            this.selectedLabel = undefined;
        }
    }
};
__decorate([
    observable
], CriteriaSegmentedControl.prototype, "criteriaOptions", void 0);
__decorate([
    observable
], CriteriaSegmentedControl.prototype, "value", void 0);
__decorate([
    observable
], CriteriaSegmentedControl.prototype, "selectedLabel", void 0);
CriteriaSegmentedControl = __decorate([
    customElement({
        name: 'criteria-segmented-control',
        template: template(),
        styles: css ``,
    })
], CriteriaSegmentedControl);
export { CriteriaSegmentedControl };
